@@ -24,6 +24,8 @@ Client*
 base_client_new(Wm *w, Window win)
 {
    XWindowAttributes attr;
+   XSizeHints        sz_hints;
+   long              mask;
 
    Client        *c = NULL;
    int            i = 0, format;
@@ -74,10 +76,10 @@ base_client_new(Wm *w, Window win)
     * Should never happen and likely throw X error to untrap.
     */
 
-   c->x      = attr.x;
-   c->y      = attr.y;
-   c->cmap   = attr.colormap;
-   c->visual = attr.visual;
+   c->x       = attr.x;
+   c->y       = attr.y;
+   c->cmap    = attr.colormap;
+   c->visual  = attr.visual;
 
    if (attr.win_gravity != NorthWestGravity)
      {
@@ -97,6 +99,14 @@ base_client_new(Wm *w, Window win)
    if (c->x < 0) c->x = (w->dpy_width + c->x - c->width);
    if (c->y < 0) c->y = (w->dpy_height + c->y - c->height);
 
+   c->gravity = NorthWestGravity;
+
+   if (XGetWMNormalHints(w->dpy, c->window, &sz_hints, &mask))
+     {
+       if (mask & PWinGravity)
+	 c->gravity = sz_hints.win_gravity;
+     }
+
 #if 0
 
    /* Get size hints */
@@ -223,6 +233,22 @@ base_client_new(Wm *w, Window win)
 
   if (data) XFree(data);
 
+  /* EWMH User time - only support value being set to 0 */
+
+  if (XGetWindowProperty(w->dpy, win,
+			 w->atoms[_NET_WM_USER_TIME], 
+			 0L, 2L, False,
+			 XA_CARDINAL, 
+			 &type, 
+			 &format,
+			 &n_items, 
+			 &bytes_after,
+			 (unsigned char **) &data) == Success
+      && n_items && data != NULL && *data == 0)
+    c->flags |= CLIENT_NO_FOCUS_ON_MAP;
+
+  if (data) XFree(data);
+
   client_get_wm_protocols(c);
 
   /* We detect any errors here to check the window hasn't dissapeared half
@@ -268,10 +294,11 @@ base_client_process_name(Client *c)
    */
 
   XTextProperty  text_prop;
+/* Patching away window numbering - Tapani 24.4.2006
   Client        *p = NULL; 
   int            i, max = 0; 
   char          *tmp_name = NULL;
-
+*/
   dbg("%s() called, name is %s\n", __func__, c->name);
    
   if (c->name == NULL)
@@ -311,7 +338,7 @@ base_client_process_name(Client *c)
     }
   
   /* If window name already exists, rename, adding <%i> to it if app window */
-
+/* Patching away window numbering - Tapani 24.4.2006
   if (!stack_empty(w) && c->type == MBCLIENT_TYPE_APP)
     {
       stack_enumerate(w, p)
@@ -343,7 +370,7 @@ base_client_process_name(Client *c)
 	  XFetchName(w->dpy, c->window, (char **)&c->name);
 	}
     }
-
+*/
    dbg("%s() end, name is now %s\n", __func__, c->name);
 }
 
@@ -438,13 +465,13 @@ base_client_destroy(Client *c)
 {
   Wm *w = c->wm;
   int i = 0;
-  Client *p = NULL;
+  Client *p = NULL, *input_method = NULL;
    /* Free its memory + remove from list */
 
    dbg("%s() called\n", __func__);
 
 
-   /* Update focus list and anythink that is transient for this */
+   /* Update focus list and anything that is transient for this */
    stack_enumerate(w, p)
      {
        if (p->next_focused_client == c)
@@ -459,7 +486,14 @@ base_client_destroy(Client *c)
 #ifdef USE_ALT_INPUT_WIN
 	 {
 	   if (p->flags & (CLIENT_TB_ALT_TRANS_FOR_DIALOG|CLIENT_TB_ALT_TRANS_FOR_APP))
-	     p->hide(p);
+	     {
+	       /* alt input methods ( maemo ) are special cased and 
+                * we need to focibly remove them. We do this later
+                * ( see below ) to be safer with tranciencys and
+                *  no removing in middle of enumeration. 
+	       */
+	       input_method = p;
+	     }
 	   else
 	     p->trans = c->trans;
 	 }
@@ -525,6 +559,18 @@ base_client_destroy(Client *c)
     ewmh_update_lists(w); 
 
     free(c);
+
+#ifdef USE_ALT_INPUT_WIN
+    if (input_method)
+      {
+	 /* were now gone */
+	input_method->trans = NULL;
+	/* Hide will destroy the client */
+	input_method->hide(input_method);
+      }
+#endif
+
+
 }
 
 
