@@ -323,6 +323,13 @@ client_set_focus(Client *c)
 	  /* Are we both transient for the same thing */
 	  if (trans_new && trans_old && trans_new == trans_old)
 	    c->next_focused_client = w->focused_client;
+
+	  /* From regular dialog to transient for root dialogs */
+	  if (w->focused_client->type == MBCLIENT_TYPE_DIALOG
+	      && c->type == MBCLIENT_TYPE_DIALOG
+	      && c->trans == NULL)
+	    c->next_focused_client = w->focused_client;
+	    
 	}
 
       if (misc_untrap_xerrors())
@@ -469,6 +476,9 @@ client_get_highest_transient(Client *c, int client_flags, Client *ignore)
 	{
 	  depth = 0;
 
+	  if (client_flags && !(p->flags & client_flags))
+	    continue;
+
 	  /* Recurse for each directly transient client, getting the depth 
 	   * for any extra matchbox children.
 	  */
