@@ -25,10 +25,13 @@
 #include "wm.h"
 
  /* Number of failed pending pings to kill a ping supporting app on */
-#define PING_PENDING_MAX 2 
+#define PING_PENDING_MAX 3 
 
  /* Time in seconds between each hung app check */
-#define PING_CHECK_FREQ  2 
+#define PING_CHECK_FREQ  4 
+
+/* Max num of pings to send to an app - used only when in aggresive mode */
+#define PING_CHECK_DURATION 5
 
 /* Non aton defines */
 #define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
@@ -68,9 +71,18 @@ ewmh_state_set(Client *c);
 Bool 
 ewmh_state_check (Client *c, Atom atom_state_wanted);
 
+int
+ewmh_get_user_time (Client *c);
+
 void 
 ewmh_set_allowed_actions (Wm *w, Client *c);
 
+void
+ewmh_ping_client_start (Client *c);
+
+void
+ewmh_ping_client_stop (Client *c);
+
 void 
 ewmh_hung_app_check (Wm *w);
 
