@@ -381,6 +381,9 @@ main_client_toggle_fullscreen(Client *c)
 {
   c->flags ^= CLIENT_FULLSCREEN_FLAG;
 
+  ewmh_state_set(c); /* Let win know it fullscreen state has changed, it
+		        could be waiting on this to adjust ui */
+
   main_client_configure(c);
   main_client_move_resize(c);
 
@@ -391,11 +394,12 @@ main_client_toggle_fullscreen(Client *c)
       c->redraw(c, False);
     }
 
-  ewmh_state_set(c); /* Let win know it fullscreen state has changed, it
-		        could be waiting on this to adjust ui */
-
   wm_activate_client(c); /* Reactivate, stacking order slightly different  */
 
+  /* No need to ping here anymore
+   * if (c->wm->config->ping_aggressive)
+   *   ewmh_ping_client_start (c);
+   */ 
 }
 
 
@@ -770,8 +774,13 @@ main_client_show(Client *c)
 {
   Wm     *w = c->wm;
 
-   dbg("%s() called on %s\n", __func__, c->name);
+  Client *visible_app_client = NULL;
+
+  dbg("%s() called on %s\n", __func__, c->name);
    
+  if (c->flags & CLIENT_NO_FOCUS_ON_MAP)
+    visible_app_client = wm_get_visible_main_client(w);
+
    if (w->flags & DESKTOP_RAISED_FLAG) 
      {
        c->flags |= CLIENT_NEW_FOR_DESKTOP;
@@ -792,7 +801,7 @@ main_client_show(Client *c)
       care of painels etc as it can use active client as a 'watermark' 
    */
    stack_move_top(c);
-   
+
    stack_dump(w);
 
    if (!c->mapped)
@@ -808,11 +817,29 @@ main_client_show(Client *c)
 	   /* Make sure any dialogs are shown too */
 	   stack_enumerate(w, p)
 	     if (p->trans == c)
-	       p->show(p);
+	       {
+		 p->show(p);
+		 /* To handle delay mapping.. */
+		 XMapSubwindows(w->dpy, p->frame);
+		 XMapWindow(w->dpy, p->frame);
+	       }
 	 }
 
-       XMapSubwindows(w->dpy, c->frame);
-       XMapWindow(w->dpy, c->frame);
+       if (c->flags & CLIENT_NO_FOCUS_ON_MAP)
+	 {
+	   if (visible_app_client)
+	     {
+	       stack_move_above_client (c, visible_app_client->below);
+	       c->flags |= CLIENT_DELAY_MAPPING;
+	     }
+	   c->flags &= ~CLIENT_NO_FOCUS_ON_MAP;
+	 }
+
+       if (!(c->flags & CLIENT_DELAY_MAPPING))
+	 {
+	   XMapSubwindows(w->dpy, c->frame);
+	   XMapWindow(w->dpy, c->frame);
+	 }
      }
 
    c->mapped = True;
