@@ -310,6 +310,7 @@ wm_load_config (Wm   *w,
    w->config->dialog_shade     = False;   
    w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
    w->config->ping_handler     = getenv("MB_HUNG_APP_HANDLER");
+   w->config->ping_aggressive = getenv("MB_AGGRESSIVE_PING") ? True : False;
 
 #ifdef USE_COMPOSITE
    w->config->dialog_shade = True;
@@ -489,6 +490,7 @@ wm_load_config (Wm   *w,
    w->config->dialog_shade     = False;   
    w->config->dialog_stratergy = WM_DIALOGS_STRATERGY_CONSTRAINED;
    w->config->ping_handler     = getenv("MB_HUNG_APP_HANDLER");
+   w->config->ping_aggressive  = getenv("MB_AGGRESSIVE_PING") ? True : False;
 
    if (XrmGetResource(rDB, "matchbox.display",
 		      "Matchbox.Display",
@@ -898,6 +900,8 @@ wm_event_loop(Wm* w)
 	    wm_handle_keypress(w, &ev.xkey); break;
 	  case PropertyNotify:
 	    wm_handle_property_change(w, &ev.xproperty); break;
+	  case GravityNotify:
+	    dbg("**** got gravity event ***"); break;
 #ifndef NO_KBD
 	  case MappingNotify:
 	    dbg("%s() got MappingNotify\n", __func__);
@@ -945,7 +949,7 @@ wm_event_loop(Wm* w)
 
 #ifndef NO_PING
 	/* check for hung apps every PING_CHECK_FREQ seconds */
-	if (++hung_app_timer > PING_CHECK_FREQ && w->n_active_ping_clients)
+	if (++hung_app_timer >= PING_CHECK_FREQ && w->n_active_ping_clients)
 	  {
 	    hung_app_timer = 0;
 	    ewmh_hung_app_check(w);
@@ -1330,8 +1334,8 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 
    dbg("%s() for win %s - have w: %i vs %i, h: %i" 
        "vs %i, x: %i vs %i, y: %i vs %i,\n", 
-       __func__, c->name, c->height, req_h, c->width, 
-       req_w, c->x, req_x, req_y, e->y );
+       __func__, c->name, c->width, req_w,
+       c->height, req_h,  c->x, req_x, c->y, req_y );
 
    /* Deal with raising - needs work, not sure if anything really
     * relies on this / or how it fits with mb. 
@@ -1452,6 +1456,38 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	 {
 	   Bool want_fake_configure = False;
 	   
+	   if (req_x == c->x && req_y == c->y 
+	       && req_w != c->width && req_h != c->height)
+	     {
+	       switch (c->gravity)
+		 {
+		 case NorthEastGravity:
+		   req_x = c->x -= (req_w - c->width);
+		   break;
+		 case EastGravity:
+		   req_x = c->x -= (req_w - c->width);
+		   break;
+		 case SouthWestGravity:
+		   req_y = c->y -= (req_h - c->height);
+		   break;
+		 case SouthGravity:
+		   req_y = c->y -= (req_h - c->height);
+		   break;
+		 case SouthEastGravity:
+		   req_x = c->x -= (req_w - c->width);
+		   req_y = c->y -= (req_h - c->height);
+		   break;
+		 case CenterGravity:
+		   /* FIXME */
+		 case NorthWestGravity:
+		 case NorthGravity:
+		 case WestGravity:
+		 case StaticGravity:
+		 default:
+		   break;
+		 }
+	     }
+
 	   dialog_constrain_geometry(c, &req_x, &req_y, &req_w, &req_h);
 
 	   if (c->width == req_w && c->height == req_h)
@@ -1469,6 +1505,8 @@ wm_handle_configure_request (Wm *w, XConfigureRequestEvent *e )
 	   xwc.x      = c->x      = req_x;
 	   xwc.y      = c->y      = req_y; 
 
+	   dbg("GRAVITY: x:%i y:%i\n", c->x, c->y);
+
 	   /* ICCCM says if window only moved - not size change,
             * then we send a fake one too.
 	    */
@@ -1751,21 +1789,19 @@ wm_handle_property_change(Wm *w, XPropertyEvent *e)
       misc_trap_xerrors(); 	/* avoid possible X Errors */
 
       c->name = (char*)ewmh_get_utf8_prop(w, c->window, w->atoms[_NET_WM_NAME]);
+      misc_untrap_xerrors();
 
-      if (!misc_untrap_xerrors())
+      if (c->name)
+	c->name_is_utf8 = True;
+      else
 	{
-	  if (c->name)
-	    c->name_is_utf8 = True;
-	  else
-	    {
-	      c->name_is_utf8 = False;
-	      XFetchName(w->dpy, c->window, (char **)&c->name);
-	    }
-
-	  base_client_process_name(c);
-	  dbg("%s() NET_WM_NAME change, name is %s\n", __func__, c->name);
-	  update_titlebar = True;
+	  c->name_is_utf8 = False;
+	  XFetchName(w->dpy, c->window, (char **)&c->name);
 	}
+
+      base_client_process_name(c);
+      dbg("%s() NET_WM_NAME change, name is %s\n", __func__, c->name);
+      update_titlebar = True;
     }
   else  if (e->atom == w->atoms[WM_PROTOCOLS])
     {
@@ -2376,6 +2412,9 @@ wm_activate_client(Client *c)
   Wm     *w;
   Client *client_to_focus = c;
   Bool    set_desktop_show_hint = False;
+#ifndef NO_PING
+  Client *prev_app_client = NULL;
+#endif
 
   if (c == NULL) return; /* its possible for this to happen :( */
 
@@ -2383,6 +2422,13 @@ wm_activate_client(Client *c)
 
   dbg("%s() called for %s\n", __func__, c->name);
 
+#ifndef NO_PING
+  if (w->config->ping_aggressive)
+    {
+      prev_app_client = wm_get_visible_main_client(w);
+    }
+#endif
+
   XGrabServer(w->dpy);
 
   c->show(c); /* Set 'relative' pos in stack, map windows
@@ -2392,6 +2438,7 @@ wm_activate_client(Client *c)
       __func__, (w->flags & DESKTOP_RAISED_FLAG));
 
 
+
   if (c->type == MBCLIENT_TYPE_APP || c->type == MBCLIENT_TYPE_DESKTOP) 
     {
       /* As matchbox works around 'main' windows ( apps/main and desktop wins).
@@ -2523,28 +2570,24 @@ wm_activate_client(Client *c)
 	}
 
       /* Check if this is transient for an app/desktop thats
-	 not currently at the top of the stack. In which case
-	 dont try and focus it.
+       * not currently at the top of the stack. In which case
+       * dont try and focus it.
       */
-      if (c->trans != NULL)     /* not trans for root */
+      if (c->trans != NULL) 	/* not trans for root */
 	{
 	  Client *dialog_parent = c->trans;
-	  
+
 	  while (dialog_parent->trans != NULL)
 	    dialog_parent = dialog_parent->trans;
-	 
-	  
+
 	  if ( (dialog_parent->type == MBCLIENT_TYPE_APP
 		|| dialog_parent->type == MBCLIENT_TYPE_DESKTOP)
 	       && wm_get_visible_main_client(w) != dialog_parent)
-	  {
-		  client_to_focus = NULL;
-	  }
-      	}
-    
-  
+	    client_to_focus = NULL;
+	}
     }
 
+
   /* Always make sure embedded titlebar panels arn't visible for desktop 
    */
   if (c == w->have_titlebar_panel 
@@ -2578,10 +2621,36 @@ wm_activate_client(Client *c)
       XMapWindow(w->dpy, c->frame);
     }
 
+
   client_set_focus(client_to_focus); /* set focus if needed and ewmh active */
 
   comp_engine_client_show(w, c);
 
+#ifndef NO_PING
+  if (w->config->ping_aggressive)
+    {
+      Client *new_app_client;
+
+      new_app_client = wm_get_visible_main_client(w);
+
+      if (prev_app_client != new_app_client)
+	{
+	  /* Only stop pings here now, activates via NET_ACTIVE
+           *
+	   * if (new_app_client 
+	   *    && new_app_client->type != MBCLIENT_TYPE_DESKTOP)
+	   *  {
+	   *   ewmh_ping_client_start (new_app_client);
+	   *  }
+	  */
+	  if (prev_app_client)
+	    {
+	      ewmh_ping_client_stop (prev_app_client);
+	    }
+	}
+    }
+#endif
+
   XSync(w->dpy, False);	    
 
   XUngrabServer(w->dpy);
